#! /usr/bin/env python
'''
Module Name:    TaskMgr.py

Description:     This is the main task manager for MUS.

Copyright:      Toshiba Confidential

                OCO Source Materials

                Copyright (C) 2012,2013 Toshiba Global Commerce Solutions, Inc.

                The source code for this program is not published
                or otherwise divested of its trade secrets,
                irrespective of what has been deposited with the
                U. S. Copyright Office.

Initial Author: Francisco Liwa

Function:

Notes:

Dependencies:    None

Restrictions:    None

Last Modified: 01 October 2013

'''
#importing core modules
import os
import sys
import subprocess
import time

#importing helper modules 

#from monitorupgrade import MUSConfigParser
from monitorupgrade import TLog as TLOG
from monitorupgrade import boot_update as bu
from monitorupgrade import Configs as Configs
from monitorupgrade import Utils as utils
from monitorupgrade import SampoVideoFwFlash as sampo
from monitorupgrade import EloTouchFwFlash as elo


#Globals
FILE_FOUND = 1
FILE_NOT_FOUND = 0
EXIT_SUCCESS = 0
EXIT_FAILURE = 1 
UNKNOWN = -1
NO_MONITORS_ATTACHED = 0

#Error Codes
ERROR_CODE_DEVICE_NOT_EXIST = 4
ERROR_CODE_OPERATION_FAILED = 11
ERROR_CODE_INVALID_PARAM = 21

#Error Strings
ERROR_STR_DEVICE_NOT_EXIST = 'Device not exist'
ERROR_STR_OPERATION_FAILED = 'Operation failed'
ERROR_STR_INVALID_PARAM = 'Invalid Parameter'
ERROR_STR_Runtime = 'Runtime error'


#sampo stuffs here
SAMPO_UTIL = '/opt/toshiba/mus/op_utils/sampo/4820MonUtil'
AFTER_LOCAL = '/etc/init.d/after.local'
RECOVERY_STR = '4820MonUtil'

#ELO Stuffs here    
ELO_DOWNLOAD = '/opt/toshiba/mus/op_utils/elo/EloDownload.sh'
ELO_CONFIG = '/opt/toshiba/mus/op_utils/elo/EloConfig.sh'
ELO_RECOVERY_FILE = '/opt/toshiba/mus/op_utils/elo/recovery.txt'
ELO_INI = '/opt/toshiba/mus/UpdateTasks/TouchConfig/EloTouch/toshiba-mus1.1_up_2xx5xx_touch-config_elo/eloConfigOpt.ini'

#helpers
LSTMON = '/usr/bin/lstmon'
RSTELO = '/usr/bin/rstelo'
CHKELOPATCH = '/usr/bin/chk_elo_patch'
CHKFOUSB = '/usr/bin/chk_fousb'
IBM_VID = '04b3'
ELO_VID = '04e7'
UP_RPMS_DIR = '/opt/toshiba/mus/RpmUpdates'


UPDATE_TYPES = ['YAM_VIDEO_FW_UPDATE',   #0
                'ANK_VIDEO_FW_UPDATE',   #1
                'YAM_TOUCH_FW_UPDATE',   #2
                'ANK_TOUCH_FW_UPDATE',   #3
                'YNK_TOUCH_CONFIG',       #4
                'DWU_VIDEO_FW_UPDATE',   #5
                'DWU_TOUCH_FW_UPDATE',   #6
                'DWU_TOUCH_CONFIG',       #7
                '3MD_TOUCH_FW_UPDATE',   #8
                '3MD_TOUCH_CONFIG',       #9
                'SPECIAL_TASK'            #10   
                ]


                
#======================================================== Main Task Manager Class ======================================================        


#==========================================================================================================
'''
TaskMgr - The main class to handle and call other helper classes for individual task
'''
#==========================================================================================================        
class TaskMgr:
    init = False
    fw_path=''
    fw_name=''
    controller=''
    retry=0
    RECOVERY_MODE = False
    mon_list = []
    video_up_tasks = []
    touch_up_tasks = []
    pending_tasks = []
    rcv_cmd = ''
    
    def __init__(self):
        '''
        Constructor
        '''
        #print "TaskMgr Constructed"
        TLOG.logger.info('TaskMgr Constructed!')

#==========================================================================================================
# initialize() - function to check if task manager has already been initialized
#==========================================================================================================
    def initialize(self):
        
        if (self.init):
            #print'TaskMgr::initialize-Already Initialized'
            TLOG.logger.info('TaskMgr::initialize-Already Initialized')
        else:
            self.init = True
            #print'TaskMgr::initialize-Initialized OK'
            self.runRpmUpdates()
            TLOG.logger.info('TaskMgr::initialize-Initialized OK')
            

#==========================================================================================================
# checkFousb() - Check if elofousb KMP driver module is installed or not
#              - if installed then exit MUS 
#              - This is for MUS 1.0 only, MUS 1.1 don't need it
#==========================================================================================================
    def checkFousb(self):
        
        res = -1
        process = subprocess.Popen(CHKFOUSB, shell=True, stdout=subprocess.PIPE)
        while True:
            line = process.stdout.readline()
            #print line
            if not line:
                break
            #TLOG.logger.info(line)
            res = int ( line.strip('\n') )    
            process.wait()
    
        return int(res)


#==========================================================================================================
# checkEloUsbPatch() - Checks whether elousb driver with fw_quirk adjustment is installed or not
#                    - If not installed  then exit MUS
#                    - This is to ensure we don't have complications with 'broken firmware' workaround
#==========================================================================================================
    def checkEloUsbPatch(self):
        
        res = -1
        process = subprocess.Popen(CHKELOPATCH, shell=True, stdout=subprocess.PIPE)
        while True:
            line = process.stdout.readline()
            #print line
            if not line:
                break
            #TLOG.logger.info(line)
            res = int( line.strip('\n') )    
            process.wait()
    
        return res

#==========================================================================================================
# doRstElo()   - Reset EloUSB devices
#==========================================================================================================
    def doRstElo(self):
        
        #res = -1
        process = subprocess.Popen(RSTELO, shell=True, stdout=subprocess.PIPE)
        while True:
            line = process.stdout.readline()
            #print line
            if not line:
                break
            #TLOG.logger.info(line)
            #res = int ( line.strip('\n') )    
            process.wait()
    
        #return int(res)

#==========================================================================================================
# runRpmUpdates() - functions to check whether we have new UP rpm files in the rpmUpdate directory
#==========================================================================================================
    def runRpmUpdates(self):
        bu.process()
#==========================================================================================================
# getMonitorList() - function to get the list of all attached monitors and its details --Video/Touch FW, etc
#==========================================================================================================
    def getMonitorList(self):
        process = subprocess.Popen('lstmon', shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE) 
        while True:
            output_line = process.stdout.readline().strip('\n')
            if not output_line:
                break
            hub,entity,vid_pid,bcdDevice,controller = output_line.split(' ')
            self.mon_list.append((hub,entity,vid_pid,bcdDevice,controller ))                       
            
            process.wait()
        return len(self.mon_list)

#==========================================================================================================
# parseFirmwareLevel() - returns the firmware level embedded on each update folders
#==========================================================================================================    
    
    def parseFirmwareLevel(self,rpm):
    
        level_str =''
        parts = rpm.split("_")
        num_parts = len(parts)
        if (num_parts>0):
            pattern = parts[0]
        if (num_parts>1):
            mus_rpm_type = parts[1]
        if (num_parts>2):
            target_models = parts[2]
        if (num_parts>3):
            update_type = parts[3]
        if (num_parts>4):
            tmp = parts[4]
            endparts = tmp.split("-")
            num_endparts = len(endparts)
            if (num_endparts>0):
                level_str = endparts[0]
        print ' FW LEVEL:', level_str
        TLOG.logger.info('TaskMgr::parseFirmwareLevel() - FW LEVEL: ' + level_str)
                
        count = len(level_str)
        fw_level = level_str[count-2:]    
        return fw_level
   
#==========================================================================================================
# updatePendingTaskList() - update the PendingTasks list based on the firmware we got
#==========================================================================================================    
    
    def updatePendingTaskList(self,task_type,fw,controller,path):
        from os import walk
        f = []
        dirpath = ''
        dirnames = ''
        filenames = ''
        # get a list of all the folders inside the UpdateTask Folder
        for (dirpath,dirnames,filenames) in walk(path):
            f.extend(filenames)
            break
        
        if fw.isdigit() == False:
            fw = int(fw,16)         #for firmware levels with bcd values, we convert them to hex
        
        new_fw = int(fw)            # keep a record of old firmware value
        
        #iterate thru the list of firmware folders
        folder_str  =''
        for folder in dirnames: 
            
            folder_name = folder
            folder=self.parseFirmwareLevel(folder)
            folder_str = folder               # hold the string representation of the folder name
          
            if folder.isdigit() == False:
                folder = int(folder, 16)      # get the hex value of the folder name for comparison  
                
            #compare the firmware of the device to the folder name corresponding to
            #available firmware updates            
            if int(folder) > int(new_fw):
                new_fw = folder
                new_fw_str = folder_str        #assign the new folder location
                name = folder_name
                print "Found! FW Level:", new_fw_str
                TLOG.logger.info('TaskMgr::updatePendingTaskList() - Found! FW Level:' + new_fw_str)
                
        if ( new_fw > int(fw) ):
            fw_path = (os.path.join(dirpath,name))
            #self.pending_tasks.append( (task_type,fw_path) )
            if(task_type == 'SampoVideoFwFlashTask'):
                self.video_up_tasks.append( (task_type,fw_path,controller) )
            elif(task_type == 'EloTouchFwFlashTask'):
                self.touch_up_tasks.append( (task_type,fw_path,controller) )
        elif ( int(new_fw) == int(fw) ):
            print 'No Firmware Update Available!'
            TLOG.logger.info('TaskMgr::updatePendingTaskList() - No Firmware Update Available!')

#==========================================================================================================
# mergeTaskList() - Merge the Video and Touch Task Lists
#==========================================================================================================               
    def mergeTaskList(self):
        for vtask in self.video_up_tasks:
            task_type,path,controller = vtask
            if task_type in (item[0] for item in self.pending_tasks) and path in (item[1] for item in self.pending_tasks):
                pass
            else:
                self.pending_tasks.append(vtask)
        for ttask in self.touch_up_tasks:
            task_type,path,controller = ttask
            if task_type in (item[0] for item in self.pending_tasks) and path in (item[1] for item in self.pending_tasks):
                pass
            else:
                self.pending_tasks.append(ttask)
             
#==========================================================================================================
# haveAvailableUpdateTasks() - Returns True if we have Update Tasks Available
#==========================================================================================================    
    def haveAvailableUpdateTasks(self):
        ret = False
        
        #get list of attached monitors
        n = self.getMonitorList()
        if ( n !=0 ):
            #check UpdateTaskFolder against mon_list 
            #iterate thru mon_list
            for index,item in enumerate(self.mon_list):
                hub,entity,vid_pid,bcdDevice,controller = item
                
                #separate the VID , PID values
                vid,pid = vid_pid.split(':') 
                
                #getting the last 2 digits of the bcdDevice for comparison
                l = list(bcdDevice)
                s = len(l)
                m  = l[s-2:s]
                fw = ''.join(m)     #folder name
                #print hub
                # check for Video FW :TV -Toshiba Video
                if ( entity == 'TV' ):
                    print '[Video Firmware]'
                    TLOG.logger.info('TaskMgr::haveAvailableUpdateTasks() -[Video Firmware]')
                    if vid == IBM_VID:
                        serial,model = controller.split(';')
                        serial = serial[2:]
                        if (pid == "4676" or pid == "4677"):  # Found Yam Monitor
                            mon_type = 'YAM'                  # Assign monitor type
                            self.updatePendingTaskList('SampoVideoFwFlashTask',fw,serial,'/opt/toshiba/mus/UpdateTasks/Video/YAM')
                                                                                  
                        elif (pid == "4678" or pid == "4679"):  # Found Ank Monitor
                            mon_type = 'ANK'                   # Assign monitor type
                            self.updatePendingTaskList('SampoVideoFwFlashTask',fw,serial,'/opt/toshiba/mus/UpdateTasks/Video/ANK')
                            
                elif (entity == 'SYV'):     # Found Yam Monitor
                    print '[Video Firmware]'
                    TLOG.logger.info('TaskMgr::haveAvailableUpdateTasks() -[Video Firmware]')
                    mon_type = 'YAM'                  # Assign monitor type
                    serial,model = controller.split(';')
                    serial = serial[2:]
                    self.updatePendingTaskList('SampoVideoFwFlashTask',fw,serial,'/opt/toshiba/mus/UpdateTasks/Video/YAM')
                                                                                  
                elif (entity == 'SAV'):  # Found Ank Monitor
                    print '[Video Firmware]'
                    TLOG.logger.info('TaskMgr::haveAvailableUpdateTasks() -[Video Firmware]')
                    mon_type = 'ANK'                   # Assign monitor type
                    serial,model = controller.split(';')
                    serial = serial[2:]
                    self.updatePendingTaskList('SampoVideoFwFlashTask',fw,serial,'/opt/toshiba/mus/UpdateTasks/Video/ANK')
                    
                elif (entity == 'TDV'):  # Dawu Monitor for MUS 2.0
                    print '[Video Firmware]'
                    TLOG.logger.info('TaskMgr::haveAvailableUpdateTasks() -[Video Firmware]')
                    mon_type = 'DWU'                   # Assign monitor type
                    serial,model = controller.split(';')
                    serial = serial[2:]
                    self.updatePendingTaskList('DawuVideoFwFlashTask',fw,serial,'/opt/toshiba/mus/UpdateTasks/Video/DWU')
                    
                # check for Touch FW : ET - ELO Touch        
                elif ( entity == 'EKT' ):
                    print '[Touch Firmware]'
                    TLOG.logger.info('TaskMgr::haveAvailableUpdateTasks() -[Touch Firmware]')
                    if vid == ELO_VID:
                        serial = controller[2:]
                        if (pid == "0009" or pid == "0030"):
                            #Check current to see if this is the Touch for the Current Monitor
                           
                            if (mon_type == 'YAM'):
                                self.updatePendingTaskList('EloTouchFwFlashTask',fw,serial,'/opt/toshiba/mus/UpdateTasks/Touch/YAM')
                            elif (mon_type == 'ANK'):
                                self.updatePendingTaskList('EloTouchFwFlashTask',fw,serial,'/opt/toshiba/mus/UpdateTasks/Touch/ANK')
                                
                elif ( entity == 'ADT' ):  # Dawu Touch Atme/Ocular
                    print '[Touch Firmware]'
                    serial = controller[2:]
                    TLOG.logger.info('TaskMgr::haveAvailableUpdateTasks() -[Touch Firmware]')                    
                    self.updatePendingTaskList('AtmelTouchFwFlashTask',fw,controller,'/opt/toshiba/mus/UpdateTasks/Touch/DWU')
                            
        else:
            TLOG.logger.info('TaskMgr::haveAvailableUpdateTasks() -No Monitors Attached.')
            print 'No Monitors Attached'
            
        self.mergeTaskList()           
        count = len(self.pending_tasks)
        if (count != 0):
            ret = True;
                    
        return ret


            

#==========================================================================================================
# checkRecoveryFile() - function to check if recovery trigger file exist for the Sampo Recovery task
#==========================================================================================================    
    def checkRecoveryFile(self,filename): 
        
        ret = FILE_NOT_FOUND
        
        TLOG.logger.info('TaskMgr::checkRecoveryFile------------------START---------------')
        TLOG.logger.info('TaskMgr::checkRecoveryFile-Checking Recovery file...')
        if utils.doesFileExist(filename) == True:
            ret = FILE_FOUND
            #print 'TaskMgr::checkRecoveryFile-Found Sampo Recovery trigger file...'
            TLOG.logger.info('TaskMgr::checkRecoveryFile-Found Recovery file...')
        else:    
            print 'TaskMgr::checkRecoveryFile-No Recovery file...'
            TLOG.logger.info('TaskMgr::checkRecoveryFile-No Recovery file...')
                    
        TLOG.logger.info('TaskMgr::checkRecoveryFile-------------------END----------------\n')
        
        return ret
        


#==========================================================================================================
# isInSampoRecoveryMode() - Returns the sampo recovery mode status of the system
#==========================================================================================================    
    def isInSampoRecoveryMode(self):
        
        TLOG.logger.info('TaskMgr::isInSampoRecoveryMode------------------START---------------') 
        TLOG.logger.info('TaskMgr::isInSampoRecoveryMode-Checking if system is in sampo recovery mode...')
        recovery = False
        cmd = ''
        
        if ( self.checkRecoveryFile(AFTER_LOCAL) == FILE_FOUND ) :  
            TLOG.logger.info('TaskMgr::isInRecoveryMode()- Checking Sampo Recovery file.')
            print  'Checking Sampo Recovery file.'  
            
            recovery_file = open(AFTER_LOCAL,'r+')
            lines = recovery_file.readlines()
            for line  in lines:
                #print line
                if(RECOVERY_STR in line):                  # We look for the string '4820MonUtil', this signifies that there is a pending recovery task
            
                    TLOG.logger.info('TaskMgr::isInRecoveryMode()- Found recovery String!- ' + line)
                    print 'Found Sampo Video Recovery file.' 
                    TLOG.logger.info('TaskMgr::isInRecoveryMode()- Proceed with Sampo Recovery Task')
                    print 'Proceed with Sampo Recovery Task'
                    
                    #ret = utils.runShell(str(line))
                    recovery = True
                    #recovery_type = 'Sampo'
                    
                    break
                
                else:
                    recovery = False
            
            recovery_file.close()
            
            self.rcv_cmd = line
        else:
                recovery =  False
                TLOG.logger.info('TaskMgr::isInSamopRecoveryMode()-FALSE')
                print 'Not in Sampo Recovery Mode...'
        
        return recovery

#==========================================================================================================
# isInEloRecoveryMode() - Returns the elo recovery mode status of the system
#==========================================================================================================       
    def isInEloRecoveryMode(self):   
          
        TLOG.logger.info('TaskMgr::isInEloRecoveryMode------------------START---------------') 
        TLOG.logger.info('TaskMgr::isInEloRecoveryMode-Checking if system is in elo recovery mode...')
        recovery = False
        cmd = ''
        
        if( self.checkRecoveryFile(ELO_RECOVERY_FILE) == FILE_FOUND ) :                  
                TLOG.logger.info('TaskMgr::isInRecoveryMode()- Checking Elo Recovery file.')
                print  'Checking Elo Touch Recovery file.'
               
                recovery_file = open(ELO_RECOVERY_FILE,'r+')
                lines = recovery_file.readlines()
                for line  in lines:
                #print line
                    if('EloRecover' in line):                  
                        
                        line = line.strip('\n')
                        tsk,cmd = line.split(' ')
                        
                        TLOG.logger.info('TaskMgr::isInRecoveryMode()- Found recovery String!- ' + line ) 
                        print 'Found Elo Touch Recovery file.'
                        TLOG.logger.info('TaskMgr::isInRecoveryMode()- Proceed with Elo Recovery Task')
                        print 'Proceed with Elo Recovery Task'
                        
                        recovery = True
                        break
                    
                recovery_file.close()
                TLOG.logger.info('TaskMgr::isInRecoveryMode - EloRecovery --------\n') 
                
                self.rcv_cmd = cmd 
        else:
                recovery =  False
                TLOG.logger.info('TaskMgr::isInEloRecoveryMode()-FALSE')
                print 'Not in Elo Recovery Mode...'
            
    
        return recovery
    
        
        
        
#==========================================================================================================
# performTasks() - The main function to be called by the MUS App/ Service to perform the actual update task
#==========================================================================================================    
    def performTasks(self):
        
        '''
            performTasks Sequence of Tasks
                     
            1. check if in Recovery mode (Sampo , ELO)
            2. check for available update tasks
            3. perform necessary update tasks
            
        ''' 
        
        TLOG.logger.info('TaskMgr::performTasks()=============================================================================================== S T A R T \n')       
        
        #recovery check
             
        if  self.isInEloRecoveryMode() == True:
            TLOG.logger.info('TaskMgr::performTasks()- Calling doEloTouchFwFlashRecoveryTask()')
            print 'Please wait...Performing Touch Firmware Recovery, the System will reboot after the recovery operation... '
           
            self.doEloTouchFwFlashRecoveryTask(self.rcv_cmd)
            utils.runShell('reboot')
            sys.exit(0)
            
        elif self.isInSampoRecoveryMode() == True:
            TLOG.logger.info('TaskMgr::performTasks()- Calling doSampoVideoFwFlashRecoveryTask()')
            print 'Please wait...Performing Video Firmware Recovery, the System will reboot after the recovery operation... '
           
            self.doSampoVideoFwFlashRecoveryTask(self.rcv_cmd)
            utils.runShell('reboot')
            sys.exit(0)
                       
      
        elif self.haveAvailableUpdateTasks() == True :
            
            TLOG.logger.info('TaskMgr::performTasks()- Found Available Update tasks...')
            
            for task in self.pending_tasks:
                
                tskname,up_path,controller = task
                TLOG.logger.info('TaskMgr::performTasks() -Task : '+ tskname)
                
                TLOG.logger.info('TaskMgr::performTasks() -Calling MUSConfigParser() ')
                
                update_config = Configs.UpdateTaskConfig(up_path)
                           
                task_type = update_config.getTaskType()              
                self.fw_path = update_config.getFirmwarePath()
                self.fw_name = update_config.getFirmwareName()
                self.controller = controller
                #print 'fw_path = ' + fw_path
                
                #Check configuration for retry
                #mus_config = Configs.MusConfig()
                #self.retry = mus_config.getRetryCount()
                
                TLOG.logger.info('TaskMgr::performTasks()- TaskName= '+ task_type)
                print 'TaskMgr::performTasks()- TaskName= '+ task_type
                
                if (tskname.strip('\n') == 'SampoVideoFwFlashTask'):
                    TLOG.logger.info('TaskMgr::performTasks()- Calling doSampoVideoFwFlashTask()')
                    
                    exit_status = self.doSampoVideoFwFlashTask(task_type)          
                    TLOG.logger.info('TaskMgr::performTasks()-'+ ' Exit Status is:' + str(exit_status) )
                    
                    if exit_status != 0:
                        TLOG.logger.info('TaskMgr::performTasks()-SampoVideoFlashTask Failed!-The System will undergo reboot to recover.' )
                        #utils.updateBootCount()
                        print 'Update task failed...The System will undergo reboot to recover.' 
                        utils.runShell('reboot')
                        raise RuntimeError("SampoVideoFlashTask Task Failed!")
                
                elif (tskname.strip('\n') == 'EloTouchFwFlashTask'):
                    TLOG.logger.info('TaskMgr::performTasks()- Calling doEloTouchFlashTask()')
                    
                    exit_status = self.doEloTouchFwFlashTask(task_type)   
                    TLOG.logger.info('TaskMgr::performTasks()-'+ ' Exit Status is:' + str(exit_status) )
                    
                    if exit_status != 0:
                        TLOG.logger.info('TaskMgr::performTasks()-EloTouchFwFlashTask Failed!-The System will undergo reboot to recover.' )
                        #utils.updateBootCount()
                        time.sleep(1)
                        print 'Update task failed...The System will undergo reboot to recover.'                      
                        utils.runShell('reboot')  
                        raise RuntimeError("EloTouchFwFlashTask Task Failed!")
#                 
#                
                                                                             
        else: #performtask
                #We don't have task available, so just exit from the method and kill the service.
                TLOG.logger.info('TaskMgr::performTasks()- No Firmware Update task done.')    
                print  'No Firmware Update task done.'   
            
            
        if (  ( len(self.mon_list)!= 0 )  and  ( utils.doesFileExist(ELO_INI) == True ) ) :
            TLOG.logger.info('TaskMgr::performTasks()- Calling doEloConfigTask()')
            exit_status = self.doEloConfigTask()
            if exit_status != 0:
                raise RuntimeError("EloConfigTask Task Failed!-exiting :" )
          
                #call rstelo
            self.doRstElo()  
        else:
            TLOG.logger.info('TaskMgr::performTasks()-EloConfigTask is not performed because either No Monitor Attached or Touch Configuration file is not found.' )
            print  'EloConfigTask is not performed because either No Monitor Attached or Touch Configuration file is not found.'
    
        TLOG.logger.info('TaskMgr::performTasks()================================================================================================== E N D \n')            
            
        return EXIT_SUCCESS   #will always return success with or without a task!
    
    
            
#==========================================================================================================
# doSampoVideoFwFlashRecoveryTask() -  function to execute SampoVideoFlashRecoveryTask run method
#==========================================================================================================
    def doSampoVideoFwFlashRecoveryTask(self,cmd):
        
        svfrt = sampo.VideoFwRecoveryFlashTask()       
        svfrt.run(cmd)     

#==========================================================================================================
# doSampoVideoFwFlashRecoveryTask() -  function to execute SampoVideoFlashRecoveryTask run method
#==========================================================================================================
    def doEloTouchFwFlashRecoveryTask(self,fw_path):
        
        etfrt = elo.TouchFwRecoveryFlashTask()       
        etfrt.run(fw_path)     


#==========================================================================================================
# doSampoVideoFwFlashTask() -  function to execute SampoVideoFlashTask run method
#==========================================================================================================
    def doSampoVideoFwFlashTask(self,tsktype):
        
        result = EXIT_SUCCESS
        if (tsktype == UPDATE_TYPES[0]):
           
            svft = sampo.VideoFwFlashTask_4820_21x51x(self.fw_path,self.controller)      
            result = svft.run()      
             
        elif (tsktype == UPDATE_TYPES[1]):     
            svft = sampo.VideoFwFlashTask_4820_2Lx5Lx(self.fw_path,self.controller)      
            result = svft.run()      
        
        
        return result
    
    
#==========================================================================================================
# do4820_2Lx5LxFlashTask() -  function to execute Elo4820_2Lx5LxFlashTask run method
#==========================================================================================================        
    def doEloTouchFwFlashTask(self,tsktype):
        
        exit_status = 0;
        
        if (tsktype == UPDATE_TYPES[2]):
            eyft = elo.TouchFwFlashTask_21x51x(self.fw_path,self.fw_name,self.controller)       
            exit_status  = eyft.run()    

        elif (tsktype == UPDATE_TYPES[3]):
            eaft = elo.TouchFwFlashTask_2Lx5Lx(self.fw_path,self.fw_name,self.controller)      
            exit_status = eaft.run()
             
        return exit_status


#==========================================================================================================
# doEloConfigTask() -  function to execute EloConfigTask run method
#==========================================================================================================    
    def doEloConfigTask(self):
        
        eCt = elo.ConfigTask()      
        result = eCt.run()
        return result

           
#==========================================================================================================
# getDeviceState() - function to get the current state of the touch device
#==========================================================================================================
    def getDeviceState(self,device_ ):
        '''
        Checking device state
        '''
        state = UNKNOWN
        deviceFound = False
        
        TLOG.logger.info('TaskMgr::getDeviceState-----------------START---------------')
                  
        if utils.doesFileExist(ELO_DOWNLOAD) != True:
                       
            print 'TaskMgr::getDeviceState()-No ELO Download script found!'
            TLOG.logger.error('TaskMgr::getDeviceState()-No ELO Download script found!')
            raise IOError('No ELO Download script found!')
        
        else:
            
            deviceFound = self.find_devices_in_fousb_mode( device_ )
            if (deviceFound == True):
                #Need to recover state of the device first:
                print  'TaskMgr::getDeviceState() :' + device_ + 'monitor needs to be recovered'
                TLOG.logger.info('TaskMgr::getDeviceState() :' + device_ + 'monitor needs to be recovered')
                state = -2
            else:
                #try to call inventory to see if any such devices are attached.        
                result = self.have_attached_display_types(device_ )
                
                if (result == NO_MONITORS_ATTACHED):
                    print  'TaskMgr::getDeviceState() :No Displays are attached at all!'
                    TLOG.logger.info('TaskMgr::getDeviceState() :No Displays are attached at all!')
                    state = -1
                    
                elif (result == True):
                    print  'TaskMgr::getDeviceState() : Has attached ' + device_ + 'display'
                    TLOG.logger.info('TaskMgr::getDeviceState() : has attached ' + device_ + 'display')
                    state = 1 
                                   
                elif (result == False):
                    print  'TaskMgr::getDeviceState() : No attached ' + device_ + 'display'
                    TLOG.logger.info('TaskMgr::getDeviceState() : No attached ' + device_ + 'display')
                    state = 2
    
        print 'TaskMgr::getDeviceState() - END '  
        TLOG.logger.info('TaskMgr::getDeviceState() -----------------END-----------------\n')
       
        return state

#==========================================================================================================
# find_devices_in_fousb_mode() - 
#==========================================================================================================
    def find_devices_in_fousb_mode(self,device_):
     
        screen = ''
        controller = ''
        result = ''
        model=''
        foundDeviceType = False
        
        find_cmd = 'elo_download -find' 
        process = subprocess.Popen(find_cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE) 
        while True:
            output_line = process.stdout.readline()
            #print output_line
            if not output_line:
                break                    
            #Response line
            if "RESPONSE" in output_line:               
                res = output_line.split()
                for r in res:
                    #print r                
                    if "SCREEN" in r:
                        idx = r.find("=")
                        for c in r[idx + 1:len(r)]:
                            if c == '\"':
                                continue
                            screen += c
                        print screen
                        TLOG.logger.info('TaskMgr::find_devices_in_fousb_mode() :screen =' + screen)
                    elif "CONTROLLER" in r:
                        idx = r.find("=")
                        for c in r[idx + 1:len(r)]:
                            if  c == '\"':
                                continue
                            controller += c
                        print controller
                        TLOG.logger.info('TaskMgr::find_devices_in_fousb_mode() :controller =' + controller)
                    elif "MODEL" in r:
                        idx = r.find("=")
                        for c in r[idx + 1:len(r)]:
                            if c == '\"' or c == '>' :
                                continue
                            model += c
                        print model
                        TLOG.logger.info('TaskMgr::find_devices_in_fousb_mode() :model =' + model)
                    else :
                        idx = r.find("=")
                        for c in r[idx + 1:len(r)]:
                            if c == '\"':
                                continue
                            result += c
                            #print result                   
            else:
                #print 'find_devices_in_fousb_mode() - No Displays in FOUSB MODE'
                TLOG.logger.info('TaskMgr::find_devices_in_fousb_mode() :- No Displays in FOUSB MODE')
    
            if device_ == model:
                print device_ + " = " + model
                foundDeviceType = True
                break
            else:
                screen=''
                controller=''
                model=''
                TLOG.logger.info('TaskMgr::find_devices_in_fousb_mode() :- No Displays in FOUSB MODE')
    
            process.wait()
    
        return foundDeviceType
         
     
#==========================================================================================================
# have_attached_display_types() - 
#==========================================================================================================
    def have_attached_display_types(self,device_ ):   
        screen = ''
        controller = ''
        result = ''
        firmware = ''
        model = '4820-'
        foundDisplayType = False
     
        elo_inventory_cmd = 'elo_download -inventory'     
        process = subprocess.Popen(elo_inventory_cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        while True:
            output_line = process.stdout.readline()
            #print output_line
            if not output_line:
                break
            #Response line
            if "RESPONSE" in output_line:
                #print output_line
                if "ERROR" in output_line:
                    print  'TaskMgr::getDeviceState() - ERROR: Controller not detected'
                    TLOG.logger.error('TaskMgr::getDeviceState() ERROR: Controller not detected')
                    return NO_MONITORS_ATTACHED
                else :
                    res = output_line.split()
                    for r in res:
                        #print r
                        if "SCREEN" in r:       
                            idx = r.find("=")
                            for c in r[idx + 1:len(r)]:
                                if c == '\"':
                                    continue
                                screen += c
                            print screen
                            TLOG.logger.info('TaskMgr::getDeviceState() :screen =' + screen)
                        elif "CONTROLLER" in r:
                            idx = r.find("=")
                            for c in r[idx + 1:len(r)]:
                                if  c == '\"':
                                    continue
                                controller += c
                            #print controller
                            TLOG.logger.info('TaskMgr::getDeviceState() :controller =' + controller)
                        #elif "MODEL" in r:
                        elif "4820" in r:
                            idx = r.find("-")
                            for c in r[idx + 1:len(r)]:
                                if c == '\"' or c == '>' :
                                    continue
                                model += c
                            #print model
                            TLOG.logger.info('TaskMgr::getDeviceState() :model =' + model)
                        else :
                            idx = r.find("=")
                            for c in r[idx + 1:len(r)]:
                                if c == '\"':
                                    continue
                                result += c
                                #print result
            elif "DATA" in output_line:
                idx = output_line.find(">")
                for c in output_line[idx + 1:len(output_line)]:
                    if c == '<':
                        break
                    firmware += c
                #print firmware 
                TLOG.logger.info('TaskMgr::getDeviceState() :firmware =' + firmware)
                
            if device_ == model:
                print device_ + " = " + model
                foundDisplayType = True                  
                break
            else:
                screen = ''
                controller = ''
                result = ''
                firmware = ''
                model = '4820-'
            
            process.wait()

        return foundDisplayType 
    

#==========================================================================================================
'''
TouchMonitor - abstraction of the touch monitor 
'''
#==========================================================================================================

class TouchMonitor(object):
    def __init__(self):
        '''
        Constructor
        '''
               
        TLOG.logger.info('TouchMonitor Constructed')
       
        #device controller id
        self.controllerId = ''
        self.modelNumber = ''
        self.touchFirmwareLevel = ''
        self.videoFirmwareLevel = ''
        self.screenNumber = ''
        self.serialNumber =''
               
    def setTouchControllerId(self,controller):
        self.controllerId = controller
    
    def setVideoSerialNumber(self,serial):
        self.serial = serial
            
    def setModelNo(self,model):
        self.modelNumber = model

    def setTouchFwLevel(self,fwlevel):
        self.touchFirmwareLevel = fwlevel
        
    def setVouchFwLevel(self,fwlevel):
        self.videoFirmwareLevel = fwlevel
    
    def setScreenNo(self,scrn_no):
        self.screenNumber = scrn_no
                    
    def getControllerId(self):
        
        controller = self.controllerId    
        TLOG.logger.info('TouchMonitor::getControllerId() - controller id = ' + controller  )
        
        return controller
    
    def getVideoSerialNumber(self):
        
            
        TLOG.logger.info('TouchMonitor::getControllerId() - controller id = ' + self.serialNumber )
        
        return self.screenNumber
   
    def getModelNumber(self):
        
        model = self.modelNumber    
        TLOG.logger.info('TouchMonitor::getModelNumber() - model = ' + model  )
        
        return model

    def getTouchFirmwareLevel(self):
        
        fw_level = self.touchFirmwareLevel    
        TLOG.logger.info('TouchMonitor::getFirmwareLevel() - fw_level = ' + fw_level  )
        
        return fw_level
    
    def getVideoFirmwareLevel(self):
        
        fw_level = self.videoFirmwareLevel    
        TLOG.logger.info('TouchMonitor::getFirmwareLevel() - fw_level = ' + fw_level  )
        
        return fw_level

    def getScreenNumber(self):
        
        screen = self.screenNumber
        TLOG.logger.info('TouchMonitor::getScreenNumber() - screen# = ' + screen  )
        
        return screen
    
    def printInfo(self):
        print 'touch controllerId =' + self.controllerId
        print 'video serial# =' + self.serialNumber
        print 'modelNumber = ' + self.modelNumber
        print 'touch firmware = ' + self.touchFirmwareLevel
        print 'video firmware = ' + self.videoFirmwareLevel
        print 'screen = ' + self.screenNumber

